package dev.kske.chess.uci;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>UCIReceiver.java</strong><br>
 * Created: <strong>19.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong>
 */
public class UCIReceiver implements Runnable {

	private final BufferedReader in;

	private UCIListener listener;

	public UCIReceiver(InputStream in) {
		this.in = new BufferedReader(new InputStreamReader(in));
	}

	@Override
	public void run() {
		try {
			while (in.ready())
				parse(in.readLine());
		} catch (IndexOutOfBoundsException ex) {
			System.err.println("Too few arguments were provided!");
			ex.printStackTrace();
		} catch (IOException ex) {
			ex.printStackTrace();
		}
	}

	private void parse(String line) {
		String command = line.substring(0, line.indexOf(' '));
		switch (command) {
			case "id":
				parseId(line.substring(command.length() + 1));
				break;
			case "uciok":
				listener.onUCIOk();
				break;
			case "readyok":
				listener.onReadyOk();
				break;
			// TODO: bestmove
			case "copyprotection":
				parseCopyProtection(line.substring(command.length() + 1));
				break;
			case "registration":
				parseRegistration(line.substring(command.length() + 1));
				break;
			// TODO: info
			// TODO: option
			default:
				System.err.printf("Unknown command '%s' found!%n", command);
		}
	}

	private void parseId(String line) {
		String	param	= line.substring(line.indexOf(' '));
		String	arg		= line.substring(param.length() + 1);
		switch (line) {
			case "name":
				listener.onIdName(arg);
				break;
			case "author":
				listener.onIdAuthor(arg);
				break;
			default:
				System.err.printf("Unknown parameter '%s' for command 'id' found!%n", param);
		}
	}

	private void parseCopyProtection(String line) {
		switch (line) {
			case "checking":
				listener.onCopyProtectionChecking();
				break;
			case "ok":
				listener.onCopyProtectionOk();
				break;
			case "error":
				listener.onCopyProtectionError();
				break;
			default:
				System.err.printf("Unknown parameter '%s' for command 'copyprotection' found!%n", line);
		}
	}

	private void parseRegistration(String line) {
		switch (line) {
			case "checking":
				listener.onRegistrationChecking();
				break;
			case "ok":
				listener.onRegistrationOk();
				break;
			case "error":
				listener.onRegistrationError();
				break;
			default:
				System.err.printf("Unknown parameter '%s' for command 'registration' found!%n", line);
		}
	}

	public void setListener(UCIListener listener) { this.listener = listener; }
}
